"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const underscore_1 = require("../../common-libs/underscore");
class CFSBlockchainArchive {
    constructor(cfs, _chunkSize) {
        this.cfs = cfs;
        this._chunkSize = _chunkSize;
    }
    async archive(records) {
        if (!this.checkBlocksRepresentChunks(records)) {
            return 0;
        }
        if (!this.checkBlocksAreWellChained(records)) {
            return 0;
        }
        const chunks = this.splitIntoChunks(records);
        for (const c of chunks) {
            const fileName = this.getFileNameForBlock(c[0].number);
            await this.cfs.writeJSON(fileName, c);
        }
        return chunks.length;
    }
    checkBlocksRepresentChunks(records) {
        return !(records[0].number % this._chunkSize !== 0 || (records[records.length - 1].number + 1) % this._chunkSize !== 0);
    }
    checkBlocksAreWellChained(records) {
        let previous = {
            number: records[0].number - 1,
            hash: records[0].previousHash,
            previousHash: ''
        };
        for (const b of records) {
            if (b.previousHash !== previous.hash || b.number !== previous.number + 1) {
                return false;
            }
            previous = b;
        }
        return true;
    }
    splitIntoChunks(records) {
        const nbChunks = records.length / this._chunkSize;
        const chunks = [];
        for (let i = 0; i < nbChunks; i++) {
            chunks.push(records.slice(i * this._chunkSize, (i + 1) * this._chunkSize));
        }
        return chunks;
    }
    async getBlock(number, hash) {
        const block = await this.getBlockByNumber(number);
        if (!block) {
            return null;
        }
        return block.hash === hash ? block : null;
    }
    async getBlockByNumber(number) {
        if (number < 0) {
            return null;
        }
        const content = await this.getChunkForBlock(number);
        if (!content) {
            // The block's chunk is not archived
            return null;
        }
        return content[this.getPositionInChunk(number)];
    }
    async getBlocks(start, end) {
        const chunkStart = this.getChunkNumber(start);
        const chunkLast = this.getChunkNumber(end);
        const chunkRange = underscore_1.Underscore.range(chunkStart, chunkLast + 1);
        const chunks = await Promise.all(chunkRange.map(c => this.getChunk(c)));
        const startInFirst = start - chunkStart * this._chunkSize;
        const endInLast = end % this._chunkSize + 1;
        let blocks = [];
        for (let i = 0; i < chunks.length; i++) {
            let toConcat = [];
            const chunk = chunks[i];
            if (chunk) {
                if (i === 0) {
                    toConcat = chunk.slice(startInFirst, chunkStart === chunkLast ? endInLast : this._chunkSize);
                }
                else if (i === chunks.length - 1) {
                    toConcat = chunk.slice(0, endInLast);
                }
                else {
                    toConcat = chunk.slice();
                }
            }
            blocks = blocks.concat(toConcat);
        }
        return blocks;
    }
    async getChunk(number) {
        const file = this.getFileNameForChunk(number);
        return this.cfs.readJSON(file);
    }
    async getChunkForBlock(number) {
        const file = this.getFileNameForBlock(number);
        return this.cfs.readJSON(file);
    }
    async getLastSavedBlock() {
        const list = await this.cfs.list('/');
        const max = list
            .map(f => f.replace(`chunk_`, ''))
            .map(f => f.replace(`-${this._chunkSize}.json`, ''))
            .map(f => parseInt(f))
            .reduce((v, max) => {
            return Math.max(v, max);
        }, 0);
        const content = await this.getChunkForBlock(max * this._chunkSize);
        if (!content) {
            return null;
        }
        return this.getBlock(content[content.length - 1].number, content[content.length - 1].hash);
    }
    getFileNameForChunk(number) {
        return CFSBlockchainArchive.getChunkName(number, this._chunkSize);
    }
    getFileNameForBlock(number) {
        const chunk = this.getChunkNumber(number);
        return CFSBlockchainArchive.getChunkName(chunk, this._chunkSize);
    }
    getChunkNumber(number) {
        const rest = number % this._chunkSize;
        return (number - rest) / this._chunkSize;
    }
    static getChunkName(chunkNumber, chunkSize) {
        return `chunk_${chunkNumber}-${chunkSize}.json`;
    }
    getPositionInChunk(number) {
        return number % this._chunkSize;
    }
    async init() {
        return this.cfs.makeTree('/');
    }
    async close() {
    }
    triggerInit() {
        // TODO: remove triggerInit from all the DAOs, it is a wrong implementation
    }
    cleanCache() {
        // TODO: is it really useful?
    }
    get chunkSize() {
        return this._chunkSize;
    }
}
exports.CFSBlockchainArchive = CFSBlockchainArchive;
//# sourceMappingURL=CFSBlockchainArchive.js.map